"""
**Deal** is a Python library for [design by contract][wiki] (DbC) programming.
See [documentation][docs] for more details.

[wiki]: https://en.wikipedia.org/wiki/Design_by_contract
[docs]: https://deal.readthedocs.io/index.html
"""

# main package info
__title__ = 'deal'
__version__ = '4.7.1'
__author__ = 'Gram Orsinium'
__license__ = 'MIT'


from ._aliases import (
    chain, ensure, has, implies, inv, post, pre, pure, raises, reason, safe,
)
from ._exceptions import (
    ContractError, InvContractError, MarkerError, OfflineContractError, PostContractError,
    PreContractError, RaisesContractError, ReasonContractError, SilentContractError,
)
from ._imports import activate, module_load
from ._schemes import Scheme
from ._state import disable, enable, reset
from ._testing import TestCase, cases


__all__ = [
    'cases',
    'Scheme',
    'TestCase',

    # state
    'disable',
    'enable',
    'reset',

    # decorators
    'chain',
    'ensure',
    'has',
    'inv',
    'post',
    'pre',
    'raises',
    'reason',

    # aliases
    'safe',
    'pure',
    'implies',

    # module level
    'module_load',
    'activate',

    # exceptions
    'ContractError',
    'InvContractError',
    'MarkerError',
    'OfflineContractError',
    'PostContractError',
    'PreContractError',
    'RaisesContractError',
    'ReasonContractError',
    'SilentContractError',
]



from typing import Iterator, Type

from .._cached_property import cached_property
from .._exceptions import ReasonContractError
from .._types import ExceptionType
from .base import Base, CallableType, SLOTS


class Reason(Base[CallableType]):
    __slots__ = SLOTS + ['event']

    @classmethod
    def _default_exception(cls) -> ExceptionType:
        return ReasonContractError

    def __init__(self, event: Type[Exception], *args, **kwargs):
        """
        Step 1. Set allowed exceptions list.
        """
        self.event = event
        super().__init__(*args, **kwargs)

    @cached_property
    def exception_type(self) -> Type[Exception]:
        if isinstance(self.exception, Exception):
            return type(self.exception)
        return self.exception

    def patched_function(self, *args, **kwargs):
        """
        Step 3. Wrapped function calling.
        """
        try:
            return self.function(*args, **kwargs)
        except self.event as origin:
            try:
                self.validate(*args, **kwargs)
            except self.exception_type:
                raise self.exception from origin
            raise

    async def async_patched_function(self, *args, **kwargs):
        """
        Step 3. Wrapped function calling.
        """
        try:
            return await self.function(*args, **kwargs)
        except self.event as origin:
            try:
                self.validate(*args, **kwargs)
            except self.exception_type:
                raise self.exception from origin
            raise

    def patched_generator(self, *args, **kwargs) -> Iterator:
        """
        Step 3. Wrapped function calling.
        """
        try:
            yield from self.function(*args, **kwargs)
        except self.event as origin:
            try:
                self.validate(*args, **kwargs)
            except self.exception_type:
                raise self.exception from origin
            raise



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['deal',
 'deal._cli',
 'deal._decorators',
 'deal.linter',
 'deal.linter._extractors']

package_data = \
{'': ['*'],
 'deal.linter': ['stubs/*',
                 'stubs/cpython/*',
                 'stubs/cpython/http/*',
                 'stubs/cpython/urllib/*',
                 'stubs/dateutil/*',
                 'stubs/dateutil/parser/*',
                 'stubs/dateutil/tz/*',
                 'stubs/dateutil/zoneinfo/*',
                 'stubs/marshmallow/*',
                 'stubs/pytz/*',
                 'stubs/requests/*',
                 'stubs/urllib3/*',
                 'stubs/urllib3/contrib/*',
                 'stubs/urllib3/util/*']}

install_requires = \
['astroid', 'deal-solver', 'hypothesis', 'pygments', 'typeguard', 'vaa>=0.2.1']

extras_require = \
{'docs': ['m2r2', 'recommonmark', 'sphinx', 'sphinx-rtd-theme'],
 'lint': ['flake8', 'flake8-commas', 'flake8-quotes', 'mypy>=0.900'],
 'test': ['coverage',
          'pytest-cov',
          'isort',
          'marshmallow',
          'pytest',
          'urllib3']}

entry_points = \
{'flake8.extension': ['DEAL = deal.linter:Checker']}

setup(name='deal',
      version='4.7.1',
      description='**Deal** is a Python library for [design by contract][wiki] (DbC) programming.',
      author='Gram',
      author_email='gram@orsinium.dev',
      url='https://github.com/orsinium/deal',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.7',
     )
